///////////////////////////////////////////////////////////////////////////////
// TEDSoft GmbH Bonn 2000 -- tool source code                                //
// Alle Rechte vorbehalten                                                   //
// All rights reserved -- unless granted by written agreement.               //
// To remove this notice is not permitted.                                   //
//                                                                           //
//  www  : www.tedsoft.de                                                    //
//  mail : info@tedsoft.de                                                   //
///////////////////////////////////////////////////////////////////////////////
//
// Version 1.0 04.01.2000 sm

// Usage:
// * Create TTEDVersionInfo instance, assign Filename and call OpenVersion
// * if no VersionInfo is available or something goes wrong an ETEDVersionInfo
//   exception is raised
// * use predefined Versionstrings (see const VersionInfoStrings)
// * or use QueryValueVIS to query predefined Versionstrings via a simple type
//   TVersionInfoStrings
// * or use QueryValue to query arbitrary strings (Versionstring not found gives empty string)
// * CloseVersion to reuse instance
// * Free everything
//
// Not Tested:
// * Using different languages (Versioninfo can be stored for different languages)
//   therefor method SetLanguageIndex protected, fTranslation only private
//   If more than one langauge, one can switch to different languages via SetLanguageIndex
//
// 2002-11-25 ef
// * Bugfix in TTEDVersionInfo.QueryValue
//
// 2002-11-26 ef
// + function GetLanguageCount, to determine the length of the translation table
// * modified SetLanguageIndex to check for invalid index
// * made SetLanguageIndex public
// + new exception message exIndexOutOfBounds

{$A+,B-,G-,H+,I+,J-,M-,P-,T+,V+,X+,Z1}  // Standard-Settings neuer Code
unit uTEDVersionInfo;

interface
uses SysUtils, Classes, Windows;

type TVersionInfoStrings = (visCompanyName, visFileDescription, visFileVersion,
       visInternalName, visLegalCopyright, visOriginalFilename, visProductname,
       visProductVersion);

const VersionInfoStrings : Array [TVersionInfoStrings] of String = (
       'CompanyName', 'FileDescription', 'FileVersion',
       'InternalName', 'LegalCopyright', 'OriginalFilename', 'Productname',
       'ProductVersion');

type TTranslation = record
      langID  : WORD;
      charset  : WORD;
     end;

     TTEDVersionInfo = class
     private
       fFileName     : String;
       fTranslation  : Array of TTranslation;
       fPrefix       : String;
       fFixedInfo    : TVSFixedFileInfo;
       fActive       : Boolean;
       fVIBuffer     : Pointer;
       fVISize       : DWord;
     protected
     public
       destructor Destroy; override;
       procedure OpenVersion;
       procedure CloseVersion;   
       function GetLanguageCount: Integer;
       procedure SetLanguageIndex(const Index: Integer);
       function QueryValueVIS (const aVis : TVersionInfoStrings) : String;
       function QueryValue (const s : String) : String;
       property Active : Boolean read fActive;
       property FileName : String read fFileName write fFileName;
       property CompanyName      : String index visCompanyName      read QueryValueVIS;
       property FileDescription  : String index visFileDescription  read QueryValueVIS;
       property FileVersion      : String index visFileVersion      read QueryValueVIS;
       property InternalName     : String index visInternalName     read QueryValueVIS;
       property LegalCopyright   : String index visLegalCopyright   read QueryValueVIS;
       property OriginalFilename : String index visOriginalFilename read QueryValueVIS;
       property ProductName      : String index visProductName      read QueryValueVIS;
       property ProductVersion   : String index visProductVersion   read QueryValueVIS;
     end;

     ETEDVersionInfo = class (Exception);

implementation

resourcestring
  exFileVersionInfoSize = 'GetFileVersionInfoSize: Error: %d';
  exFileVersionInfo     = 'GetFileVersionInfo: Error: %d';
  exVerQueryValue       = 'VerQueryValue: Error reading %s';
  exTranslation         = 'Translation array smaller than 1';
  exIndexOutOfBounds    = 'SetLanguageIndex: Index out of bounds';

procedure TTEDVersionInfo.CloseVersion;
begin
  if fVIBuffer <> nil then FreeMem (fVIBuffer, fVISize);
  fVIBuffer := nil;
  fVISize   := 0;
  fActive := False;
end;

procedure TTEDVersionInfo.OpenVersion;
var Dummy        : DWord;
    pp           : Pointer;
    dwBytes      : DWord;
begin
  fVISize := GetFileVersionInfoSize(PChar (fFileName), Dummy);
  if fVISize=0
  then raise ETEDVersionInfo.CreateFmt (exFileVersionInfoSize, [GetLastError]);

  GetMem(fVIBuffer, fVISize);
  try
    if not GetFileVersionInfo(PChar (fFileName), 0, fVISize, fVIBuffer)
    then raise ETEDVersionInfo.CreateFmt (exFileVersionInfo, [GetLastError]);

    if not VerQueryValue(fVIBuffer, '\', pp, dwBytes)
    then raise ETEDVersionInfo.CreateFmt (exVerQueryValue, ['Root']);
    Move(pp^, fFixedInfo, dwBytes);

    if not VerQueryValue(fVIBuffer, '\VarFileInfo\Translation', pp, dwBytes)
    then raise ETEDVersionInfo.CreateFmt (exVerQueryValue, ['Translation']);
    SetLength (fTranslation, dwBytes div SizeOf (TTranslation));
    if Length (fTranslation) < 1
    then raise ETEDVersionInfo.Create (exTranslation);
    Move(pp^, Pointer (fTranslation)^, SizeOf (fTranslation));

    SetLanguageIndex (0);
    fActive := True;
  except
    FreeMem (fVIBuffer, fVISize);
    fActive := False;
    raise;
  end;
end;   

function TTEDVersionInfo.GetLanguageCount: Integer;
begin
  Result := Length (fTranslation);
end;

procedure TTEDVersionInfo.SetLanguageIndex (const Index : Integer);
begin
  if (Index < Low (fTranslation)) or (Index > High (fTranslation))
  then raise ETEDVersionInfo.Create (exIndexOutOfBounds);
  fPrefix := '\StringFileInfo\' + IntToHex(fTranslation[Index].langID, 4) +
              IntToHex(fTranslation[Index].charset,4) + '\';
end;

function TTEDVersionInfo.QueryValueVIS (const aVIS : TVersionInfoStrings) : String;
begin
  Result := QueryValue (VersionInfoStrings[aVIS]);
end;

function TTEDVersionInfo.QueryValue(const s: String) : String;
var dwBytes      : DWord;
    pp           : Pointer;
begin
  Result := '';
  if Active then begin
    if VerQueryValue(fVIBuffer, PChar (fPrefix + s), pp, dwBytes) then begin
      //if dwBytes>0 then begin
      if dwBytes>1 then begin // EF: 25.11.02
        SetLength (Result, dwBytes - 1);
        Move(pp^, Result[1], dwBytes - 1);
      end;
    end;
  end;
end;

destructor TTEDVersionInfo.Destroy;
begin
  if Active then CloseVersion;
  inherited Destroy;
end;

end.
